with Ada.Unchecked_Deallocation;
with Ada.Text_IO; use Ada.Text_IO;
procedure Access_Examples is

   type Integer_Ptr is access Integer;
   type String_Ptr  is access String;

   N : Integer_Ptr;
   S : String_Ptr;

   type Node_Type;
   type Node_Ptr is access Node_Type;
   type Node_Type is
      record
         Info : Character;
         Next : Node_Ptr;
      end record;

   Top : Node_Ptr;  -- Designates the top node in the stack
   To_Recyle : Node_Ptr;

   procedure Free is new Ada.Unchecked_Deallocation
                                       (Object => Node_Type, -- The object type
                                        Name   => Node_Ptr); -- The pointer type

   type G_String_Ptr is access all String;

   procedure Free is new Ada.Unchecked_Deallocation
                                       (Object => String,        -- The object type
                                        Name   => G_String_Ptr); -- The pointer type
   Flower : G_String_Ptr;

   Value : Character;
   Weed : aliased String := "Dandelion";

begin
   N := new Integer;        -- Obtain memory for an integer
   S := new String (1..7);  -- Obtain memory for a string

   N.all := 42;             -- Assign 42 to the memory designated by N
   S.all := "Mildred";      -- Assign "Mildred" to the memory designated by S

   N := new Integer'(42);       -- Obtain memory for an integer with initial value
   S := new String'("Mildred"); -- Obtain memory for a string with initial value

   -- Push 'd' onto the stack
   Top := new Node_Type'(Info => 'd',
                         Next => Top);

   -- Pop a character from the stack
   To_Recyle := Top;      -- Make To_Recycle an alias of Top
   Value := Top.all.Info; -- Copy the information from the top node
   Top   := Top.all.Next; -- Unlink the top node from the linked list
   Free (To_Recyle);      -- Deallocate the popped node's memory


   Flower := Weed'Access;     -- Assign the location of Weed to Flower
   Put_Line (Flower.all);     -- Displays Dandelion
   Flower.all := "Pissenlit"; -- Change the value of Weed
   Put_Line (Weed);           -- Displays Pissenlit

   Flower := new String'("Daffodil");
   Put_Line (Flower.all);
   Free (Flower); -- Free was instantiated from Unchecked_Deallocation
end Access_Examples;