with System.Storage_Elements;  use  System.Storage_Elements;
package body ADC is

   use Common_Units;  -- For operators on Volts

   -- Addresses of the ADC's two registers (memory mapped I/O)
   CSR_Address  : constant System.Address :=
                           To_Address (16#60008010#);
   Data_Address : constant System.Address :=
                           To_Address (16#60008012#);

   -- Ten milliseconds
   Max_Conversion_Time : constant Duration := 10 * 1.0E-3;

   type CSR_Rec is  -- Type for the control status register
      record
         Start      : Boolean;
         Done       : Boolean;
         Int_Enable : Boolean;
         Overflow   : Boolean;
         Gain       : Gain_Choice;
         Channel    : Channel_Number;
      end record;

--   for CSR_Rec use
--      record
--         Start      at 0 range 0 .. 0;
--         Done       at 0 range 1 .. 1;
--         Int_Enable at 0 range 2 .. 2;
--         Overflow   at 0 range 3 .. 3;
--         Gain       at 1 range 0 .. 2;
--         Channel    at 1 range 3 .. 7;
--      end record;

   for CSR_Rec use
      record
         Start      at 0 range 0 .. 0;
         Done       at 0 range 1 .. 1;
         Int_Enable at 0 range 2 .. 2;
         Overflow   at 0 range 3 .. 3;
         Gain       at 0 range 8 .. 10;
         Channel    at 0 range 11 .. 15;
      end record;

   for CSR_Rec'Size use 16;
   for CSR_Rec'Bit_Order use System.Low_Order_First;

   type Data_Type is range 0 .. 4095; -- for the data register
   for Data_Type'Size use 16;

   -- The two device registers

   Control_Status_Register : CSR_Rec;
   for Control_Status_Register'Address use CSR_Address;
   for Control_Status_Register'Size use 16;
   pragma Volatile (Control_Status_Register);

   Data_Register : Data_Type;
   for Data_Register'Address use Data_Address;
   for Data_Register'Size use 16;
   pragma Volatile (Data_Register);

   ---------------------------------------------
   procedure Read (Channel : in  Channel_Number;
                   Gain    : in  Gain_Choice;
                   Value   : out Input_Volts) is
      Shadow : CSR_Rec;
   begin
      -- Set up for the conversion (use a record aggregate)
      Shadow := (Start      => True,  Done     => False,
                 Int_Enable => False, Overflow => False,
                 Gain       => Gain,  Channel   => Channel);
      -- Start the conversion (by setting the start bit)
      Control_Status_Register := Shadow;
      -- Wait until the conversion is complete
      loop
         delay Max_Conversion_Time / 10;
         exit when Control_Status_Register.Done;
      end loop;
      if Control_Status_Register.Overflow then
         raise Over_Range;
      end if;
      -- Convert whole number data to a fixed point voltage
      Value := 5 * Volts (Data_Register) / 4095;
   end Read;

end ADC;
