with Ada.Text_IO;  use Ada.Text_IO;
procedure Array_Examples is

   type Index_Type      is range 1..1000;
   type Inventory_Array is array (Index_Type) of Natural;

   subtype Lowercase is Character range 'a' .. 'z';
   type    Percent   is range 0..100;
   type    Frequency_Array is array (Lowercase) of Percent;

   type Day_Type is (Monday, Tuesday, Wednesday, Thursday,
                     Friday, Saturday, Sunday);
   type On_Call_Array is array (Day_Type) of Boolean;

   Inventory : Inventory_Array;
   Control   : Frequency_Array;
   Unknown   : Frequency_Array;
   On_Call   : On_Call_Array;

   Total_Days : Natural;

   type Row_Index is range 1..1000;
   type Col_Index is range -5..+5;

   type Two_D_Array is array (Row_Index, Col_Index) of Float;

   type One_Row is array (Col_Index) of Float;
   type Another_2D_Array is array (Row_Index) of One_Row;

   Canary : Two_D_Array;      -- A 2 dimensional array variable
   Finch  : Another_2D_Array; -- An array of arrays variable

   type Float_Array is array (Positive range <>) of Float;
   subtype Small_Array is Float_Array (1..10);
   subtype Large_Array is Float_Array (1000..9999);

   Small : Small_Array; -- An array of 10 Float values
   Large : Large_Array; -- An array of 9,000 Float values

   function Average (Values : in Float_Array ) return Float is
      Sum : Float;
   begin
      Sum := 0.0;
      for Index in Values'Range loop
         Sum := Sum + Values (Index);
      end loop;
      return Sum / Float (Values'Length);
   end Average;

   Avg : Float;


begin
   Inventory (5)    := 1_234;
   Control ('a')    := 2 * Control ('a');
   On_Call (Sunday) := False;

   Total_Days := 0;
   for Day in Day_Type loop
      if On_Call (Day) then
         Total_Days := Total_Days + 1;
      end if;
   end loop;

   Unknown := Control;

   if Control = Unknown then
      Put ("The values in the two arrays are identical");
   elsif Control < Unknown then
      Put ("The values in Control come lexicographically before those in Unknown");
   end if;

   -- Copy elements 11-20 into locations 1-10
   Inventory (1 .. 10) := Inventory (11 .. 20);

   -- Copy elements 2-11 into locations 1-10
   Inventory (1 .. 10) := Inventory (2 .. 11);

   Canary (12, 2) := 0.0;  -- Assign zero to row 12, column 2 of the 2-D array
   Finch (12)(2)  := 0.0;  -- Assign zero to the second component of the 12th array

   Large (1001..1010) := Small;                    -- Copy 10 values
   if Small /= Large (2001..2010) then             -- Compare 10 values
      Large (2001..2021) := Small & 14.2 & Small;  -- Copy 21 values
   end if;

   Avg := Average (Small);              -- The average of 10 values
   Avg := Average (Large);              -- The average of 1000 values
   Avg := Average (Large (2001..2010)); -- The average of 10 values
   Avg := Average (Large & Small);      -- The average of 1010 values
end Array_Examples;