package Bounded_Queue_V1 is
-- Version 1, details of the queue type are not encapsulated

   subtype Element_Type is Integer;

   type Queue_Array is array (Positive range <>) of Element_Type;
   type Queue_Type (Max_Size : Positive) is
      record
        Count : Natural  := 0;   -- Number of items
        Front : Positive := 1;   -- Index of first item
        Rear  : Positive := Max_Size;  -- Index of last item
        Items : Queue_Array (1 .. Max_Size);  -- The element array
      end record;

   Overflow  : exception;
   Underflow : exception;

   procedure Clear (Queue : in out Queue_Type);

   procedure Enqueue (Queue : in out Queue_Type;
                      Item  : in     Element_Type);
   -- Overflow is raised on attempt to Enqueue an element
   --          onto a full queue.  Queue is unchanged.

   procedure Dequeue (Queue : in out Queue_Type;
                      Item  :    out Element_Type);
   -- Underflow is raised on attempt to dequeue an element
   --           from an empty Queue.  Queue remains empty.

   function Full (Queue : in Queue_Type) return Boolean;

   function Empty (Queue : in Queue_Type) return Boolean;
end Bounded_Queue_V1;