with System.Machine_Code; use System.Machine_Code;
package body Port_IO is

   -- The x86 IN and OUT instructions are accessed through
   -- the ASM function found in package System.Machine_Code.

   -- For all routines, we use register DX to hold the port
   -- address.  This avoids the hassles of determining if the
   -- value is in 0..255, where we can use immediate mode.

   procedure Out_Byte (Address : in Address_Range;
                       Data    : in Byte) is
   begin
      System.Machine_Code.Asm
        (Template => "outb %%al, %%dx",
         Outputs  =>  No_Output_Operands,
         Inputs   => (Byte'Asm_Input ("a", Data),
                      Address_Range'Asm_Input ("d", Address)),
         Clobber  => "",
         Volatile => True);

   end Out_Byte;

   procedure Out_Word (Address : in Address_Range;
                       Data    : in Word) is
   begin
      System.Machine_Code.Asm
        (Template => "outw %%ax, %%dx",
         Outputs  =>  No_Output_Operands,
         Inputs   => (Word'Asm_Input ("a", Data),
                      Address_Range'Asm_Input ("d", Address)),
         Clobber  => "",
         Volatile => True);

   end Out_Word;

   function In_Byte (Address : in Address_Range) return Byte is
      Result : Byte;
   begin
      System.Machine_Code.Asm
        (Template => "inb %%dx, %%al",
         Outputs  => Byte'Asm_Output ("=a", Result),
         Inputs   => Address_Range'Asm_Input ("d", Address),
         Clobber  => "",
         Volatile => True);

      return Result;
   end In_Byte;

   function In_Word (Address : in Address_Range) return Word is
      Result : Word;
   begin
      System.Machine_Code.Asm
        (Template => "inw %%dx, %%ax",
         Outputs  => Word'Asm_Output ("=a", Result),
         Inputs   => Address_Range'Asm_Input ("d", Address),
         Clobber  => "",
         Volatile => True);

      return Result;
   end In_Word;
end Port_IO;
