with Turnstile;            use Turnstile;
with Ada.Exceptions;
with Ada.Integer_Text_IO;  use Ada.Integer_Text_IO;
with Ada.Text_IO;          use Ada.Text_IO;
procedure Museum is

-- This program uses a protected object shared by five
-- tasks to monitor the number of visitors inside a museum

   Maximum : constant := 100;  -- The Museum capacity

   protected Population is -- The current number of visitors
      procedure Increment;
      procedure Decrement;
      function  Current return Natural;
   private
      Count : Natural  := 0;
   end Population;

   protected body Population is
      procedure Increment is
      begin
         Count := Count + 1;
      end Increment;

      procedure Decrement is
      begin
         Count := Count - 1;
      end Decrement;

      function Current return Natural is
      begin
         return Count;
      end Current;
   end Population;

   -- A task type for monitoring turnstiles
   task type Entrance_Monitor
             (My_Entrance : Turnstile.Entrance_Type);

   task body Entrance_Monitor is
      Direction : Turnstile.Direction_Type;
   begin
      loop
         -- Wait until someone passes through my turnstile
         Turnstile.Get (My_Entrance, Direction);

         -- Update the number of people in the Museum
         case Direction is
            when Turnstile.Enter =>
               Population.Increment;
            when Turnstile.Leave =>
               Population.Decrement;
         end case;
      end loop;
   exception
      when Except : others =>
         Put_Line
           (File => Standard_Error,
            Item => Entrance_Type'Image (My_Entrance) &
               " turnstile task terminated with exception "
               & Ada.Exceptions.Exception_Name (Except));
   end Entrance_Monitor;

   -- One task to monitor each of the four museum doors
   North_Door : Entrance_Monitor (Turnstile.North);
   South_Door : Entrance_Monitor (Turnstile.South);
   East_Door  : Entrance_Monitor (Turnstile.East);
   West_Door  : Entrance_Monitor (Turnstile.West);

   Museum_Full   : Boolean := False;
   Current_Count : Natural;

begin
   loop
      Current_Count := Population.Current;
      Put ("Current number of visitors is ");
      Put (Item => Current_Count, Width => 1);
      New_Line;

      -- Check for change in ticket selling status
      if not Museum_Full and Current_Count >= Maximum then
         Put_Line ("The Museum is full.  " &
                   "Suspend ticket sales.");
         Museum_Full := True;
      elsif Museum_Full and Current_Count < Maximum then
         Put_Line ("The Museum is no longer full.  " &
                   "Resume ticket sales.");
         Museum_Full := False;
      end if;

      delay 1.0;  -- Update status every second
   end loop;
end Museum;
