with Turnstile;            use Turnstile;
with Ada.Exceptions;
with Ada.Integer_Text_IO;  use Ada.Integer_Text_IO;
with Ada.Text_IO;          use Ada.Text_IO;
procedure Museum_V1 is

-- This program uses a global variable shared by five tasks
-- to monitor the number of visitors inside a museum
--
-- This version works "most of the time"
--
--     Sometimes at closing time, it indicates that
--     the museum is not empty when indeed it is empty
--
--     Sometimes an entrance monitor task terminates with
--     the exception Constraint_Error

   Maximum     : constant := 100; -- Museum capacity
   Population  : Natural  := 0; -- Current number of visitors

   -- A task type for monitoring turnstiles
   task type Entrance_Monitor
             (My_Entrance : Turnstile.Entrance_Type);

   task body Entrance_Monitor is
      Direction : Turnstile.Direction_Type;
   begin
      loop
         -- Wait until someone passes through my turnstile
         Turnstile.Get (My_Entrance, Direction);

         -- Update the number of people in the Museum
         case Direction is
            when Turnstile.Enter =>
               Population := Population + 1;
            when Turnstile.Leave =>
               Population := Population - 1;
         end case;
      end loop;
   exception
      when Except : others =>
         Put_Line
           (File => Standard_Error,
            Item => Entrance_Type'Image (My_Entrance) &
               " turnstile task terminated with exception "
               & Ada.Exceptions.Exception_Name (Except));
   end Entrance_Monitor;

   -- One task to monitor each of the four museum doors
   North_Door : Entrance_Monitor (Turnstile.North);
   South_Door : Entrance_Monitor (Turnstile.South);
   East_Door  : Entrance_Monitor (Turnstile.East);
   West_Door  : Entrance_Monitor (Turnstile.West);

   Museum_Full : Boolean := False;

begin
   loop
      Put ("Current number of visitors is ");
      Put (Item => Population, Width => 1);
      New_Line;

      -- Check for change in ticket selling status
      if not Museum_Full and Population >= Maximum then
         Put_Line ("The Museum is full.  " &
                   "Suspend ticket sales.");
         Museum_Full := True;
      elsif Museum_Full and Population < Maximum then
         Put_Line ("The Museum is no longer full.  " &
                   "Resume ticket sales.");
         Museum_Full := False;
      end if;

      delay 1.0;  -- Update status every second
   end loop;
end Museum_V1;
