with Resource_Allocation;
with Ada.Numerics.Discrete_Random;
with Ada.Numerics.Float_Random;
with Protected_Output;
procedure Resource_Allocation_Demo is

   subtype Request_Range is Integer range 1 .. 8;

   package Random_Num is new Ada.Numerics.Discrete_Random (Request_Range);

   -- The resource manager
   Manager : Resource_Allocation.Manager (Initial_Inventory => 10);

   ------------------------------------------------------------------
   protected Random_Request is
      procedure Get (Number : out Request_Range);
      -- Returns a random request number
   private
      First_Call   : Boolean := True;
      My_Generator : Random_Num.Generator;
   end Random_Request;

   protected body Random_Request is
      procedure Get (Number : out Request_Range) is
      begin
         if First_Call then
            -- On the first call, reset the generator
            Random_Num.Reset (My_Generator);
            First_Call := False;
         end if;
         -- Get a random float value and convert it to a duration
         Number := Random_Num.Random (My_Generator);
      end Get;
   end Random_Request;

   ------------------------------------------------------------------
   protected Random_Duration is
      procedure Get (Item : out Duration);
      -- Returns a random duration value between 0.0 and 2.0 seconds
   private
      First_Call   : Boolean := True;
      My_Generator : Ada.Numerics.Float_Random.Generator;
   end Random_Duration;

   protected body Random_Duration is
      procedure Get (Item : out Duration) is
      begin
         if First_Call then
            -- On the first call, reset the generator
            Ada.Numerics.Float_Random.Reset (My_Generator);
            First_Call := False;
         end if;
         -- Get a random float value and convert it to a duration
         Item := 2 * Duration (Ada.Numerics.Float_Random.Random (My_Generator));
      end Get;
   end Random_Duration;

   ------------------------------------------------------------------
   task type Requester (ID : Positive);
   task body Requester is
      My_Request : Request_Range;
      My_Delay   : Duration;
   begin
      loop
         -- Request a random number of resources
         Random_Request.Get (My_Request);
         -- Get the resources
         Manager.Take (My_Request);
         Protected_Output.Put_Line ("Task" & Positive'Image (ID) & " got "
                                    & Integer'Image (My_Request));
         Random_Duration.Get (My_Delay);
         delay My_Delay;
         -- Replace the resources
         Manager.Replace (My_Request);
         Random_Duration.Get (My_Delay);
         delay My_Delay;
      end loop;
   end Requester;

   -- A number of requester tasks
   R1 : Requester (1);
   R2 : Requester (2);
   R3 : Requester (3);
   R4 : Requester (4);


begin
   null;
end Resource_Allocation_Demo;