with Ada.Text_IO;         use Ada.Text_IO;
with Ada.Float_Text_IO;   use Ada.Float_Text_IO;
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;
with Ctrl_C;              use Ctrl_C;
procedure ATC_Examples is

   subtype Non_Neg_Float is Float range 0.0 .. Float'Last;

   -- Under Windows, it is necessary to add a
   -- delay 0.0 statement within the loop in order
   -- for the scheduler to abort the loop

   function Sqrt (X     : in Non_Neg_Float;
                  Limit : in Duration)     return Float is
      Approx : Float := 0.5 * X;
   begin
      select
         delay Limit;
      then abort
         loop
            Approx := 0.5 * (Approx + X / Approx);
            delay 0.0;
         end loop;
      end select;
      return Approx;
   end Sqrt;


   Value : Integer;

begin

   -- The clock under Windows is not as fine grained as we
   -- would like.  Notice the different values from
   -- different runs of the same code.
   Put (Item => Sqrt (X => 12345.6789, Limit => 0.001),
        Fore => 1, Aft => 12, Exp => 0);
   New_Line;
   Put (Item => Sqrt (X => 12345.6789, Limit => 0.005),
        Fore => 1, Aft => 12, Exp => 0);
   New_Line;
   Put (Item => Sqrt (X => 12345.6789, Limit => 0.010),
        Fore => 1, Aft => 12, Exp => 0);
   New_Line;
   Put (Item => Sqrt (X => 12345.6789, Limit => 0.020),
        Fore => 1, Aft => 12, Exp => 0);
   New_Line;
   Put (Item => Sqrt (X => 12345.6789, Limit => 0.030),
        Fore => 1, Aft => 12, Exp => 0);
   New_Line;

   select
      Ctrl_C_Interrupt.Wait;
      Put_Line ("Handled Ctrl C");
   then abort
      loop
         Put_Line ("Enter an integer (Ctrl C to exit)");
         Get (Value);
         Put (Value);
         New_Line;
      end loop;
   end select;
   Skip_Line;  -- hold AdaGIDE window open
end ATC_Examples;
   