with Ada.Text_IO; use Ada.Text_IO;
with Ada.Numerics.Float_Random;
procedure Coffee_Vending is

   --------------------------------------------------------
   procedure Run_Coffee_System (Sugar : in Boolean;
                                Cream : in Boolean) is
   begin
      Put_Line ("Making coffee");
      if Sugar then
         Put_Line ("   with sugar");
      end if;
      if Cream then
         Put_Line ("   with cream");
      end if;
      delay 0.2;
   end Run_Coffee_System;

   procedure Sanitize_Coffee_System is
   begin
      Put_Line ("Sanitizing coffee system");
      delay 0.2;
   end Sanitize_Coffee_System;

   --------------------------------------------------------
   procedure Run_Tea_System (Sugar : in Boolean;
                             Milk  : in Boolean;
                             Lemon : in Boolean) is
   begin
      Put_Line ("Making tea");
      if Sugar then
         Put_Line ("   with sugar");
      end if;
      if Milk then
         Put_Line ("   with milk");
      end if;
      if Lemon then
         Put_Line ("   with lemon");
      end if;
      delay 0.2;
   end Run_Tea_System;

   procedure Sanitize_Tea_System is
   begin
      Put_Line ("Sanitizing tea system");
      delay 0.2;
   end Sanitize_Tea_System;

   --------------------------------------------------------
   procedure Run_Cocoa_System  is
   begin
      Put_Line ("Making cocoa");
      delay 0.2;
   end Run_Cocoa_System;

   procedure Sanitize_Cocoa_System is
   begin
      Put_Line ("Sanitizing cocoa system");
      delay 0.2;
   end Sanitize_Cocoa_System;

   --------------------------------------------------------
   procedure Sanitize_Dispenser_System is
   begin
      Put_Line ("Sanitizing dispenser system");
      New_Line;
      delay 0.2;
   end Sanitize_Dispenser_System;


   --------------------------------------------------------
   task Make_Beverage is
      entry Coffee (Sugar : in Boolean;
                    Cream : in Boolean);
      entry Tea (Sugar : in Boolean;
                 Milk  : in Boolean;
                 Lemon : in Boolean);
      entry Cocoa;
   end Make_Beverage;

   task body Make_Beverage is
      With_Sugar : Boolean;
      With_Cream : Boolean;
      With_Milk  : Boolean;
      With_Lemon : Boolean;
   begin
      loop
         select
            accept Coffee (Sugar : in Boolean;
                           Cream : in Boolean) do
               With_Sugar := Sugar;
               With_Cream := Cream;
            end Coffee;
            Run_Coffee_System (With_Sugar, With_Cream);
            Sanitize_Coffee_System;
         or
            accept Tea (Sugar : in Boolean;
                        Milk  : in Boolean;
                        Lemon : in Boolean) do
               With_Sugar := Sugar;
               With_Milk  := Milk;
               With_Lemon := Lemon;
            end Tea;
            Run_Tea_System (With_Sugar, With_Milk, With_Lemon);
            Sanitize_Tea_System;
         or
            accept Cocoa;
            Run_Cocoa_System;
            Sanitize_Cocoa_System;
         or
            terminate;
         end select;
         Sanitize_Dispenser_System;
      end loop;
   end Make_Beverage;

   ------------------------------------------------------------------
   protected Random_Duration is
      procedure Get (Item : out Duration);
      -- Returns a random duration value between 0.0 and 1.0 seconds
   private
      First_Call   : Boolean := True;
      My_Generator : Ada.Numerics.Float_Random.Generator;
   end Random_Duration;

   protected body Random_Duration is
      procedure Get (Item : out Duration) is
      begin
         if First_Call then
            -- On the first call, reset the generator
            Ada.Numerics.Float_Random.Reset (My_Generator);
            First_Call := False;
         end if;
         -- Get a random float value and convert it to a duration
         Item := Duration (Ada.Numerics.Float_Random.Random (My_Generator));
      end Get;
   end Random_Duration;

   My_Delay : Duration;

begin
   -- Simulate beverage orders
   for Count in 1 .. 10 loop
      Random_Duration.Get (My_Delay);
      if My_Delay <= 0.333333 then
         Make_Beverage.Coffee (True, False);
      elsif My_Delay <= 0.666667 then
         Make_Beverage.Tea (True, False, True);
      else
         Make_Beverage.Cocoa;
      end if;
      delay My_Delay / 2;
   end loop;
end Coffee_Vending;

