with Ada.Text_IO;       use Ada.Text_IO;
with Ada.Float_Text_IO; use Ada.Float_Text_IO;
procedure Furnace is

   subtype Degrees is Float range 32.0 .. 85.0;

   Current : Degrees := 60.0;
   On      : Boolean := False;

   procedure Get_Temperature (Item : out Degrees) is
   begin
      if On then
         Current := Current + 0.15;
      else
         Current := Current - 0.1;
      end if;
      Item := Current;
      Put (Item => Current, Fore => 3, Aft => 1, Exp => 0);
      New_Line;
   end Get_Temperature;

   procedure Turn_Furnace_On is
   begin
      Put_Line ("Furnace is on.");
      On := True;
   end Turn_Furnace_On;

   procedure Turn_Furnace_Off is
   begin
      Put_Line ("Furnace is off.");
      On := False;
   end Turn_Furnace_Off;

   task Thermostat is
      entry Enable (Set_Point : in Degrees);
      entry Disable;
   end Thermostat;

   task body Thermostat is
      Current    : Degrees; -- Current house temperature
      Target     : Degrees; -- Desired house temperature
      Furnace_On : Boolean := False; -- Flag for guards
   begin
      accept Enable (Set_Point : in Degrees) do
         Target := Set_Point;
      end Enable;
      loop
         select
            accept Enable (Set_Point : in Degrees) do
               Target := Set_Point;
            end Enable;
         or
            accept Disable;
            Turn_Furnace_Off;
            Furnace_On := False;
            accept Enable (Set_Point : in Degrees) do
               Target := Set_Point;
            end Enable;
         or
            when Furnace_On =>
            delay 1.50;
         or
            when not Furnace_On =>
            delay 0.50;
         end select;
         Get_Temperature (Current);
         if Furnace_On and Current >= Target + 0.5 then
            Turn_Furnace_Off;
            Furnace_On := False;
         elsif not Furnace_On and Current <= Target - 0.5 then
            Turn_Furnace_On;
            Furnace_On := True;
         end if;
      end loop;
   end Thermostat;

begin
   Current := 66.2;
   Thermostat.Enable (68.0);
   Put_Line ("Press enter to disable furnace.");
   Skip_Line;
   Thermostat.Disable;
   -- Attempt to disable an already disabled thermostat
   select
      Thermostat.Disable;  -- Attempt to rendezvous
      Put_Line ("Disabled the thermostat");
   or
      delay 5.0; -- Wait for 5 seconds for rendezvous
      Put_Line ("Gave up waiting for rendezvous");
   end select;

   -- Attempt to disable an already disabled thermostat
   select
      Thermostat.Disable;  -- Attempt to rendezvous
      Put_Line ("Disabled the thermostat");
   else
      Put_Line ("Could not rendezvous right away");
   end select;

   Put_Line ("Press enter to reenable furnace.");
   Skip_Line;
   Current := 66.0;
   Thermostat.Enable (68.0);
end Furnace;

      