with Lights;  -- Package with light control/status registers
package body Headlights_V2 is

   procedure Both_Off is
   begin
      Lights.Set_Headlights_Off;
      Lights.Set_High_Beams_Off;
   end Both_Off;

   type Effect_Ptr is access procedure;

   L_Off : constant Effect_Ptr := Lights.Set_Headlights_Off'Access;
   L_On  : constant Effect_Ptr := Lights.Set_Headlights_On'Access;
   H_Off : constant Effect_Ptr := Lights.Set_High_Beams_Off'Access;
   H_On  : constant Effect_Ptr := Lights.Set_High_Beams_On'Access;
   B_Off : constant Effect_Ptr := Both_Off'Access;

   type Table_Entry is
      record
         Next_State : State_Type;
         Effect     : Effect_Ptr;
      end record;
   type Transition_Array is array (State_Type, Event_Type)
                               of Table_Entry;

   Transition_Table : constant Transition_Array :=
    (((On,   L_On), (Off, null),  (Off,  null), (Off, null)),
     ((On,   null), (Off, L_Off), (High, H_On), (On,  null)),
     ((High, null), (Off, B_Off), (High, null), (On,  H_Off)));

   protected Control is
      procedure Signal_Event (Event : Event_Type);
      function Current_State return State_Type;
   private
      State : State_Type := Off;
   end Control;

   protected body Control is
      procedure Signal_Event (Event : Event_Type) is
      begin
         -- Carry out any effect for the state change
         if Transition_Table (State, Event).Effect /= null then
            Transition_Table (State, Event).Effect.all;
         end if;
         -- Determine the next state from the table
         State := Transition_Table (State, Event).Next_State;
      end Signal_Event;

      function Current_State return State_Type is
      begin
         return State;
      end Current_State;
   end Control;

   -- External Operations ----------------------------------
   procedure Signal_Event (Event : Event_Type) is
   begin
      Control.Signal_Event (Event);
   end Signal_Event;

   function Current_State return State_Type is
   begin
      return Control.Current_State;
   end Current_State;

end Headlights_V2;
