with Induction_Motor.Operations; use Induction_Motor.Operations;
package body Induction_Motor is

   protected Motor is -- For the shared state
      procedure Set_State (To : in State_Type);
      function Current_State return State_Type;
   private
      State : State_Type := Off;
   end Motor;

   protected body Motor is
      procedure Set_State (To : in State_Type) is
      begin
         State := To;
      end Set_State;
      function Current_State return State_Type is
      begin
         return State;
      end Current_State;
   end  Motor;

   --------------------------------------------------------
   task Control is
      entry Switch_On;
      entry Switch_Off;
   end Control;

   task body Control is
   begin
      loop
         case Motor.Current_State is
            when Off =>
               select
                  accept Switch_Off;
               or
                  accept Switch_On;
                  Motor.Set_State (To => Starting);
                  Operations.Set_Start_Windings (To => On);
               end select;
            when Starting =>
               select
                  accept Switch_Off;
                  Motor.Set_State (To => Off);
                  Operations.Set_Start_Windings (To => Off);
               or
                  accept Switch_On;
               or
                  delay 60.0; -- for motor to get up to speed
                  Motor.Set_State (To => Running);
                  Operations.Set_Start_Windings (To => Off);
                  Operations.Set_Run_Windings   (To => On);
               end select;
            when Running =>
               select
                  accept Switch_Off;
                  Motor.Set_State (To => Off);
                  Operations.Set_Run_Windings (To => Off);
               or
                  accept Switch_On;
               or
                  delay 1.0; -- Check temperature once a second
                  if Operations.Current_Temperature > Max then
                     Motor.Set_State (To => Off);
                     Operations.Set_Run_Windings (To => Off);
                  end if;
               end select;
         end case;
      end loop;
   end Control;

   -- External Operations ---------------------------------
   procedure Switch_On is
   begin
      Control.Switch_On;
   end Switch_On;

   procedure Switch_Off is
   begin
      Control.Switch_Off;
   end Switch_Off;

   function Current_State return State_Type is
   begin
      return Motor.Current_State;
   end Current_State;

end Induction_Motor;
