with POSIX;                        use POSIX;
with POSIX.Signals;                use POSIX.Signals;
with POSIX.Timers;                 use POSIX.Timers;
with POSIX.Process_Identification; use POSIX.Process_Identification;
with POSIX.Process_Scheduling;     use POSIX.Process_Scheduling;
procedure Generic_Periodic_POSIX_Process is

   Info       : POSIX.Signals.Signal_Info;
   Waited_Set : POSIX.Signals.Signal_Set;
   Event      : POSIX.Signals.Signal_Event;
   Timerid    : POSIX.Timers.Timer_Id;
   Timerdata  : POSIX.Timers.Timer_State;
   Options    : POSIX.Timers.Timer_Options;

   Pid    : Process_ID;
   Sched  : Scheduling_Parameters;
   Period : POSIX.Timespec;

begin

   -- Set up signal management
   POSIX.Signals.Set_Signal (Event => Event,
                             Sig   => SIGUSR1);
   POSIX.Signals.Set_Notification
         (Event  => Event,
          Notify => POSIX.Signals.Signal_Notification);
   POSIX.Signals.Delete_All_Signals (Waited_Set);
   POSIX.Signals.Add_Signal (Set => Waited_Set,
                             SIG => SIGUSR1);

   -- Create and initialize the timer to the period
   Timerid := POSIX.Timers.Create_Timer
                 (Clock => POSIX.Timers.Clock_Realtime,
                  Event => Event);
   Period := POSIX.To_Timespec (S  => Second_Period,
                                NS => Nanosecond_Period);
   POSIX.Timers.Set_Initial
      (State  => Timerdata,
       Initial => Period);
   POSIX.Timers.Set_Interval
      (State => Timerdata,
       Interval => Period);
   POSIX.Timers.Arm_Timer
      (Timerid, Options, Timerdata);

   -- Change priority and policy
   Pid   := Get_Process_Id;
   Sched := Get_Scheduling_Parameters (Pid);
   Set_Priority (Parameters => Sched,
                 Priority   => Priority_Parameter);
   Set_Scheduling_Policy (Process    => Pid,
                          New_Policy => Policy_Parameter,
                          Parameters => Sched);
   Set_Scheduling_Parameters (Process    => Pid,
                              Parameters => Sched);

   -- The periodic process
   loop
      Run; -- Call the procedure with the code for this job
      Info := POSIX.Signals.Await_Signal (Waited_Set);
   end loop;
end Generic_Periodic_POSIX_Process;

