with POSIX;    use POSIX;
with POSIX.Process_Scheduling;
   use POSIX.Process_Scheduling;
with POSIX.Process_Identification;
   use POSIX.Process_Identification;
with POSIX.Unsafe_Process_Primitives;
with Ada.Text_IO;    use Ada.Text_IO;
with Generic_Periodic_Posix_Process;
procedure Posix_Car_Example is

   -- The follow three parameterless procedures are stubs for
   -- the actual code that runs in the car

   procedure Read_Speed is
   begin
   	Put_Line("Tspeed reads speed sensor");
   end Read_Speed;

   procedure Monitor_Engine is
   begin
   	Put_Line("Tengine performs engine monitoring algorithm");
   end Monitor_Engine;

   procedure Display_Speed is
   begin
    Put_Line("Tdisplay displays the speed of the car");
   end Display_Speed;

   -- Instantiate an "infinite loop" procedure
   -- for each of our three proceses

   procedure Tdisplay is new Generic_Periodic_POSIX_Process
       (Run                => Display_Speed,
        Priority_Parameter => 12,
        Policy_Parameter   => SCHED_FIFO,
        Nanosecond_Period  => 1_000_000,
        Second_Period      => 0);
   procedure Tspeed is new Generic_Periodic_POSIX_Process
       (Run                => Read_Speed,
        Priority_Parameter => 11,
        Policy_Parameter   => SCHED_FIFO,
        Nanosecond_Period  => 2_500_000,
        Second_Period      => 0);
   procedure Tengine is new Generic_Periodic_POSIX_Process
       (Run                => Monitor_Engine,
        Priority_Parameter => 10,
        Policy_Parameter   => SCHED_FIFO,
        Nanosecond_Period  => 5_000_000,
        Second_Period      => 0);

   Pid : Process_ID;
   Sched  : Scheduling_Parameters;

begin

   -- Priority for "environment procedure"
   -- (for main POSIX process)
   Pid   := Get_Process_Id;
   Sched := Get_Scheduling_Parameters (Pid);
   Set_Priority (Parameters => Sched,
                 Priority   => 20);
   Set_Scheduling_Policy (Process    => Pid,
                          New_Policy => SCHED_FIFO,
                          Parameters => Sched);
   Set_Scheduling_Parameters (Process    => Pid,
                              Parameters => Sched);


   -- The main process creates a new child process
   Pid := POSIX.Unsafe_Process_Primitives.Fork;

   if Pid = Null_Process_ID then -- the child process?
      Tengine; -- Execute the engine process
   else
      -- The main process creates a second child
      Pid := POSIX.Unsafe_Process_Primitives.Fork;

      if Pid = Null_Process_ID then -- the child process?
         Tdisplay; -- Execute the display process
      else
         -- The main process creates a third child
         Pid:= POSIX.Unsafe_Process_Primitives.Fork;

         if Pid = Null_Process_ID then -- the child process?
            Tspeed;  -- Execute the speed process
         end if;
      end if;
   end if;

   -- All child processes start to work after the
   -- higher priority main process completes!
   Put_Line ("Processes will start after " &
             "the main process completes!");
end Posix_Car_Example;


