with Ada.Text_IO;                  use Ada.Text_IO;
with Ada.Integer_Text_IO;          use Ada.Integer_Text_IO;
with POSIX.Process_Identification; use POSIX.Process_Identification;
with POSIX.Process_Scheduling;     use POSIX.Process_Scheduling;
procedure Posix_Scheduling_Example is

   procedure Display_Policy (Policy : in Scheduling_Policy) is
   begin
      if Policy = SCHED_RR then
         Put ("/SCHED_RR");
      elsif Policy = SCHED_OTHER then
         Put("/SCHED_OTHER");
      elsif Policy = SCHED_FIFO then
         Put("/SCHED_FIFO");
      else
         Put("/unknown policy");
      end if;
   end Display_Policy;

    Pid   : Process_ID;
    Sched : Scheduling_Parameters;

begin
   -- Read current process unique identifcator
   Pid:=Get_Process_Id;

   -- Display the default priority and scheduling parameters
   -- of a process in the current operating system
   Sched := Get_Scheduling_Parameters (Pid);
   Put ("Default priority/policy on this POSIX operating system : ");
   Put (Get_Priority (Sched));
   Display_Policy (Policy => Get_Scheduling_Policy (Pid));
   New_Line;

   -- Change both priority and policy
   -- Apply SCHED_FIFO with a priority level of 10
   Set_Priority (Parameters => Sched,
                 Priority   => 10);
   Set_Scheduling_Policy (Process    => Pid,
                          New_Policy => SCHED_FIFO,
                          Parameters => Sched);
   Set_Scheduling_Parameters (Process    => Pid,
                              Parameters => Sched);

   -- Display the updated priority and scheduling parameters
   Put ("Updated priority/policy : ");
   Put (Get_Priority(Sched));
   Display_Policy (Policy => Get_Scheduling_Policy (Pid));
   New_Line;
end Posix_Scheduling_Example;

