with POSIX;         use POSIX;
with POSIX.Signals; use POSIX.Signals;
with POSIX.Timers;  use POSIX.Timers;
with Text_IO;       use Text_IO;
procedure Posix_Timer_Example is

   -- The job carried out by the periodic process
   procedure Run_Periodic_Job_Of_The_Process is
   begin
      Put_Line("Periodic task is released and executes its job");
   end Run_Periodic_Job_Of_The_Process;

   Info       : POSIX.Signals.Signal_Info;
   Waited_Set : POSIX.Signals.Signal_Set;
   Event      : POSIX.Signals.Signal_Event;
   Timerid    : POSIX.Timers.Timer_Id;
   Timerdata  : POSIX.Timers.Timer_State;
   Options    : POSIX.Timers.Timer_Options;

   Nanosecond_Period : Nanoseconds := 0;
   Second_Period     : Seconds     := 1;
   Period            : POSIX.Timespec;

begin

   -- Set up signal management

   POSIX.Signals.Set_Signal
      (Event => Event,
       Sig   => POSIX.Signals.SIGUSR1);
   POSIX.Signals.Set_Notification
      (Event  => Event,
       Notify => POSIX.Signals.Signal_Notification);
   POSIX.Signals.Delete_All_Signals (Waited_Set);
   POSIX.Signals.Add_Signal
      (Set => Waited_Set,
       Sig => POSIX.Signals.SIGUSR1);

   -- Create and initialize the timer to the period of the process

   Timerid := POSIX.Timers.Create_Timer
                 (Clock => POSIX.Timers.Clock_Realtime,
                  Event => Event);

   Period := POSIX.To_Timespec (S  => Second_Period,
                                NS => Nanosecond_Period);
   POSIX.Timers.Set_Initial
      (State   => Timerdata,
       Initial => Period);
   POSIX.Timers.Set_Interval
      (State    => Timerdata,
       Interval => Period);
   POSIX.Timers.Arm_Timer
      (Timer     => Timerid,
       Options   => Options,
       New_State => Timerdata);

   -- The periodic process
   loop
      -- Block until the signal is received
      Info := POSIX.Signals.Await_Signal (Waited_Set);
      -- Carry out the job of the periodic process
      Run_Periodic_Job_Of_The_Process;
   end loop;

end Posix_Timer_Example;

